@@ -4,17 +4,21 @@ import logging
 from copy import copy
 from dataclasses import dataclass
 from io import StringIO
+from types import ModuleType
 
 from typing import Union, TextIO, Type, Optional, List, Dict
 
+import click
 import yaml
 from hbreader import FileInfo, hbread
 from jsonasobj2 import as_dict
+from linkml.generators.pythongen import PythonGenerator
 from linkml.utils import datautils
-from linkml.utils.datautils import infer_index_slot
+from linkml.utils.datautils import infer_index_slot, _is_xsv, get_dumper
 from linkml_runtime import SchemaView
-from linkml_runtime.linkml_model import SchemaDefinition
+from linkml_runtime.linkml_model import SchemaDefinition, ClassDefinition, SlotDefinition
 from linkml_runtime.loaders import json_loader, yaml_loader, csv_loader, rdf_loader, rdflib_loader
+from linkml_runtime.utils.compile_python import compile_python
 
 from linkml_runtime.utils.yamlutils import YAMLRoot, DupCheckYamlLoader
 
@@ -89,14 +93,14 @@ def load_structured_file(source: Union[str, dict, TextIO], target_class: Union[s
         else:
             schemaview = SchemaView(schema)
             schema = schemaview.schema
-    if target_class is None:
+    #if target_class is None:
         # TODO: de-convolute this
         # attempt to infer root class, but if this cannot
         # be found then we rely on there being a type designator present
-        try:
-            target_class = datautils.infer_root_class(schemaview)
-        except:
-            pass
+        #try:
+        #    target_class = datautils.infer_root_class(schemaview)
+        #except:
+        #    pass
     if target_class and isinstance(target_class, str):
         target_class = python_module.__dict__[target_class]
     if isinstance(source, str):
@@ -113,7 +117,7 @@ def load_structured_file(source: Union[str, dict, TextIO], target_class: Union[s
         else:
             obj = load_from_csv(source, delimiter=delimiter)
             logging.debug(f'INST {target_class} <= {obj}')
-            out_obj = [instantiate_object(x, target_class, python_module=python_module) for x in obj]
+            out_obj = [instantiate_object(x, target_class, python_module=python_module, schemaview=schemaview) for x in obj]
     elif datautils._is_rdf_format(fmt):
         loader = rdflib_loader
         out_obj = rdflib_loader.load(source, schemaview=schemaview, fmt=fmt)
@@ -130,9 +134,9 @@ def load_structured_file(source: Union[str, dict, TextIO], target_class: Union[s
             loader = json_loader
             loader.json_clean(obj)
         if isinstance(obj, list):
-            out_obj = [instantiate_object(x, target_class, type_designator=type_designator, python_module=python_module) for x in obj]
+            out_obj = [instantiate_object(x, target_class, type_designator=type_designator, python_module=python_module, schemaview=schemaview) for x in obj]
         else:
-            out_obj = instantiate_object(obj, target_class, type_designator=type_designator, python_module=python_module)
+            out_obj = instantiate_object(obj, target_class, type_designator=type_designator, python_module=python_module, schemaview=schemaview)
     else:
         raise ValueError(f'Unknown format: {fmt}')
     logging.info(f'Loaded {type(out_obj)} using {loader}')
@@ -140,15 +144,20 @@ def load_structured_file(source: Union[str, dict, TextIO], target_class: Union[s
 
 def instantiate_object(data: Dict, target_class: Type[YAMLRoot] = None, schemaview: SchemaView = None,
                        type_designator: str = '@type',
-                       python_module=None) -> YAMLRoot:
+                       python_module: ModuleType = None) -> YAMLRoot:
     #if not(target_class or schemaview):
     #    raise ValueError(f'Must pass AT LEAST ONE OF target_class OR schemaview')
     if not target_class:
         if type_designator in data:
             target_class_name = data[type_designator]
+            if ':' in target_class_name:
+                pfx, loc = str(target_class_name).split(':')
+                target_class_name = loc
             data = copy(data)
         else:
-            raise ValueError(f'Cannot determine type for {data}')
+            target_class_name = datautils.infer_root_class(schemaview)
+            if target_class_name is None:
+                raise ValueError(f'Cannot determine type for {data}')
         target_class = python_module.__dict__[target_class_name]
         if '@type' in data:
             del data['@type']
@@ -166,7 +175,13 @@ def get_type_designator(schemaview: SchemaView = None) -> str:
     else:
         return '@type'
 
-
+def add_index_slot(sv: SchemaView) -> None:
+    container = ClassDefinition('Container', from_schema='http://x.org/')
+    for cn in sv.all_classes():
+        a = SlotDefinition(cn, multivalued=True, range=cn, inlined=True, inlined_as_list=True)
+        container.attributes[a.name] = a
+    sv.add_class(container)
+    sv.set_modified()
 
 
 
